@@ -26,8 +26,6 @@
 
 #include <config.h>
 
-#define _LARGEFILE64_SOURCE
-
 #include <libgnomevfs/gnome-vfs-cancellation.h>
 #include <libgnomevfs/gnome-vfs-context.h>
 #include <libgnomevfs/gnome-vfs-i18n.h>
@@ -54,11 +52,8 @@
 #include <string.h>
 #ifdef HAVE_FAM
 #include <fam.h>
+#include <glib/giochannel.h>
 #endif
-#include <gdk/gdk.h>
-
-
-/*#include "file-method.h"*/
 
 #ifdef HAVE_FAM
 FAMConnection *fam_connection = NULL;
@@ -67,6 +62,7 @@ G_LOCK_DEFINE_STATIC (fam_connection);
 typedef struct {
 	FAMRequest request;
 	GnomeVFSURI *uri;
+	gboolean     cancelled;
 } FileMonitorHandle;
 
 #endif
@@ -608,7 +604,8 @@ get_stat_info (GnomeVFSFileInfo *file_info,
 	gboolean is_symlink;
 	gboolean recursive;
 	char *link_file_path;
-
+	char *symlink_name;
+	
 	followed_symlink = FALSE;
 	
 	recursive = FALSE;
@@ -645,6 +642,7 @@ get_stat_info (GnomeVFSFileInfo *file_info,
 	gnome_vfs_stat_to_file_info (file_info, statptr);
 
 	if (is_symlink) {
+		symlink_name = NULL;
 		link_file_path = g_strdup (full_name);
 		
 		/* We will either successfully read the link name or return
@@ -658,13 +656,16 @@ get_stat_info (GnomeVFSFileInfo *file_info,
 			 * far as we can.
 			 */
 
-			file_info->symlink_name = read_link (link_file_path);
-			if (file_info->symlink_name == NULL) {
+			g_free (symlink_name);
+			symlink_name = read_link (link_file_path);
+			if (symlink_name == NULL) {
 				g_free (link_file_path);
 				return gnome_vfs_result_from_errno ();
 			}
 			
 			if ((options & GNOME_VFS_FILE_INFO_FOLLOW_LINKS) == 0
+			                /* if we had an earlier ELOOP, don't get in an infinite loop here */
+			        || recursive
 					/* we don't care to follow links */
 				|| lstat (file_info->symlink_name, statptr) != 0
 					/* we can't make out where this points to */
@@ -673,9 +674,11 @@ get_stat_info (GnomeVFSFileInfo *file_info,
 				break;
 			}
 			g_free (link_file_path);
-			link_file_path = g_strdup (file_info->symlink_name);
+			link_file_path = g_strdup (symlink_name);
 		}
 		g_free (link_file_path);
+
+		file_info->symlink_name = symlink_name;
 	}
 
 	return GNOME_VFS_OK;
@@ -759,7 +762,14 @@ do_read_directory (GnomeVFSMethod *method,
 
 	handle = (DirectoryHandle *) method_handle;
 	
+	errno = 0;
 	if (readdir_r (handle->dir, handle->current_entry, &result) != 0) {
+		/* Work around a Solaris bug.
+		 * readdir64_r returns -1 instead of 0 at EOF.
+		 */
+		if (errno == 0) {
+			return GNOME_VFS_ERROR_EOF;
+		}
 		return gnome_vfs_result_from_errno ();
 	}
 	
@@ -780,6 +790,10 @@ do_read_directory (GnomeVFSMethod *method,
 		return GNOME_VFS_OK;
 	}
 	
+	if (handle->options & GNOME_VFS_FILE_INFO_GET_MIME_TYPE) {
+		get_mime_type (file_info, full_name, handle->options, &statbuf);
+	}
+
 	return GNOME_VFS_OK;
 }
 
@@ -1980,13 +1994,13 @@ do_set_file_info (GnomeVFSMethod *method,
 }
 
 #ifdef HAVE_FAM
-static void
-fam_callback (gpointer data, 
-	      gint fd, 
-	      GdkInputCondition condition)
+static gboolean
+fam_callback (GIOChannel *source,
+	      GIOCondition condition,
+	      gpointer data)
 {
+	gboolean cancelled;
 	FileMonitorHandle *handle;
-	GnomeVFSURI *info_uri = NULL;
 	GnomeVFSMonitorEventType event_type = GNOME_VFS_MONITOR_EVENT_CHANGED;
 
 	G_LOCK (fam_connection);
@@ -1994,19 +2008,16 @@ fam_callback (gpointer data,
 	while (FAMPending(fam_connection)) {
 		FAMEvent ev;
 		if (FAMNextEvent(fam_connection, &ev) != 1) {
-			gdk_input_remove(fd);
 			FAMClose(fam_connection);
 			g_free(fam_connection);
 			fam_connection = FALSE;
 			G_UNLOCK (fam_connection);
-			return;
+			return FALSE;
 		}
 
 		handle = (FileMonitorHandle *)ev.userdata;
 
-		info_uri = gnome_vfs_uri_append_file_name (handle->uri, 
-				ev.filename);
-
+		cancelled = handle->cancelled;
 		switch (ev.code) {
 			case FAMChanged:
 				event_type = GNOME_VFS_MONITOR_EVENT_CHANGED;
@@ -2026,16 +2037,26 @@ fam_callback (gpointer data,
 				event_type = GNOME_VFS_MONITOR_EVENT_CREATED;
 				break;
 			case FAMAcknowledge:
+				/* I am not clear on the exact sematics of the Acknowledge
+				 * do we get this for all changes or just cancel ?
+				 * Be defensive for now.
+				 */
+				if (handle->cancelled) {
+					gnome_vfs_uri_unref (handle->uri);
+					g_free (handle);
+				}
+				/*fall through, and ignore */
+
 			case FAMExists:
 			case FAMEndExist:
 			case FAMMoved:
 				event_type = -1;
-				g_warning ("bad FAM event");
 				break;
 		}
 
-
-		if (event_type != -1) {
+		if (event_type != -1 && !cancelled) {
+			GnomeVFSURI *info_uri = gnome_vfs_uri_append_file_name (
+				handle->uri, ev.filename);
 			gnome_vfs_monitor_callback (
 					(GnomeVFSMethodHandle *)handle,
 					info_uri, event_type);
@@ -2043,13 +2064,18 @@ fam_callback (gpointer data,
 	}
 
 	G_UNLOCK (fam_connection);
+
+	return TRUE;
 }
 
 
 
 static gboolean
-monitor_setup ()
+monitor_setup (void)
 {
+	GIOChannel *ioc;
+	gint watch_id;
+
 	G_LOCK (fam_connection);
 
 	if (fam_connection == NULL) {
@@ -2058,10 +2084,14 @@ monitor_setup ()
 			g_print ("FAMOpen failed, FAMErrno=%d\n", FAMErrno);
 			g_free(fam_connection);
 			fam_connection = NULL;
+			G_UNLOCK (fam_connection);
 			return FALSE;
 		}
-		gdk_input_add(FAMCONNECTION_GETFD(fam_connection), 
-				GDK_INPUT_READ, fam_callback, fam_connection);
+		ioc = g_io_channel_unix_new (FAMCONNECTION_GETFD(fam_connection));
+		watch_id = g_io_add_watch (ioc,
+					   G_IO_IN | G_IO_HUP | G_IO_ERR,
+					   fam_callback, fam_connection);
+		g_io_channel_unref (ioc);
 	}
 
 	G_UNLOCK (fam_connection);
@@ -2086,6 +2116,7 @@ do_monitor_add (GnomeVFSMethod *method,
 
 	handle = g_new0 (FileMonitorHandle, 1);
 	handle->uri = uri;
+	handle->cancelled = FALSE;
 	gnome_vfs_uri_ref (uri);
 	filename = get_path_from_uri (uri);
 	
@@ -2122,14 +2153,14 @@ do_monitor_cancel (GnomeVFSMethod *method,
 		return GNOME_VFS_ERROR_NOT_SUPPORTED;
 	}
 
+	if (handle->cancelled)
+		return GNOME_VFS_OK;
+
+	handle->cancelled = TRUE;
 	G_LOCK (fam_connection);
 	FAMCancelMonitor (fam_connection, &handle->request);
 	G_UNLOCK (fam_connection);
 
-	gnome_vfs_uri_unref (handle->uri);
-
-	g_free (handle);
-
 	return GNOME_VFS_OK;
 #else
 	return GNOME_VFS_ERROR_NOT_SUPPORTED;
